import { select, 
        csv, 
        scaleBand,
       	nest,
       	scaleOrdinal,
       	schemeCategory10,
       	descending } from 'd3';
import { colorLegend } from './colorLegend'
import { parseYear, loadAndProcessData } from './loadAndProcessData'
import { lineChart } from './lineChart'

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

/* Accesser functions that access the x and y values,
i.e. year and population. */
const xValue = d => d.year;
const yValue = d => d.population;

/* An accesser function that retrieves the value of each region.*/
const colorValue = d => d.name;

/* Declares the lineChartG and colorLegendG group element. */
const lineChartG = svg.append('g')
const colorLegendG = svg.append('g')

/* A helper function that computes the number of people
corresponding to each region so as to sort the color legend
accordingly. 
d => nested, d.values => nested.values */
const lastYValue = d => yValue(d.values[d.values.length - 1])

/* Takes the input of a year value and set it as the selected year. 
Then re-render everything. */
const setSelectedYear = year => { 
  selectedYear = year; 
  render();
}

/* The data that we are going to use in computing 
colorLegend and lineChart. */
let data;

/* Default value for the selected Year. */
let selectedYear = 2018;

const render = () => {
  const colorScale = scaleOrdinal(schemeCategory10);
  
  /* A nested data structure, with the key being each region
  (d => d.name), and values that associate with each region.
  This is used in computing the line chart and greatly
  simplifies the work. */
  const nested = nest()
  	.key(colorValue)
  	.entries(data)
  	/* This is somehow similar to how Java Comparable
    interface works. The sort function here takes in input
    of two objects a and b, compare their values, and
    sort the whole thing out based on this comparison. */
  	.sort((a, b) => descending(
    	lastYValue(a), lastYValue(b)
    ))
  
  /* Set the domain of the color scale
  to be the key value of our newly constructed array.
  That guarantees that the domain is non-repeated. */
  colorScale.domain(nested.map(d => d.key));
  
  /* A call to compute the line chart.
  Refer to the detailed steps in lineChart.js. */
  lineChartG.call(lineChart, {
  	colorScale,
    colorValue,
    xValue,
    yValue,
    title: 'Population Over Time',
  	xAxisLabel: 'Time', 
  	yAxisLabel: 'Population', 
  	circleRadius: 6, 
  	margin: { top: 50, left: 70, right: 170, bottom: 55},
    width,
    height,
    data,
    nested,
    selectedYear,
    setSelectedYear
  })
  
  /* A call to compute the color legend. */
  colorLegendG
		.attr('transform', `translate(815, 160)`)
    .call(colorLegend, {
      colorScale, 
      circleRadius: 10,
      spacing: 30,
      textOffset: 22
    });
}

/* Process the data and load the data to const "data". */
loadAndProcessData().then((loadedData) => {
	data = loadedData;
  render();
});