import { scaleTime,
 				 extent,
       	 scaleLinear, 
         axisBottom,
         axisLeft,
         line,
         curveBasis,
         format,
         mouse
       		} from 'd3';
import { parseYear } from './loadAndProcess'

export const lineChart = (selection, props) => {
	const {
    colorScale, 
    colorValue,
  	xValue, 
    yValue,
  	title,
  	xAxisLabel, 
  	yAxisLabel, 
  	circleRadius, 
  	margin,
  	width, 
  	height,
  	data,
  	nested,
  	selectedYear,
  	setSelectedYear } = props;
  
  /* Width and height for the line chart. */
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom; 
  
  /* xScale and yScale for the axis. 
  We use d3.extent to map the domain that corresponds to
  the specified range. */
  const xScale = scaleTime()
  	.domain(extent(data, xValue))
  	.range([0, innerWidth])
  	.nice();
    
  const yScale = scaleLinear()
  	.domain(extent(data, yValue))
  	.range([innerHeight, 0])
  	.nice();
    
  // The whole thing is contained in group g, as we need to have identical margins. 
  const g = selection.selectAll('.container').data([null]);
	const gEnter = g.enter().append('g')
  	.attr('class', 'container');
  gEnter.merge(g)
  	.attr('transform', `translate(${margin.left}, ${margin.top})`);
  
  const xAxis = axisBottom(xScale)
  	.tickSize(-innerHeight)
  	.tickPadding(10);
    
  const yAxisTickFormat = number =>
   format('.2s')(number)
  		.replace('G', 'B')
  		.replace('.0', '');
  
  // Here is the yAxis. 
  const yAxis = axisLeft(yScale)
  	.tickSize(-innerWidth)
  	.tickFormat(yAxisTickFormat)
  	.tickPadding(5);
  
  const yAxisGEnter = gEnter
  	.append('g')
      .attr('class', 'y-axis')
  const yAxisG = yAxisGEnter.merge(g.select('.y-axis'))
    .call(yAxis)
  	.selectAll('.domain').remove();
  
  yAxisGEnter.append('text')
  	.attr('transform', `rotate(-90)`)
  	.attr('class', 'axis-label')
  	.attr('y', -37)
  	.attr('fill', 'black')
  	.style('text-anchor', 'middle')
  	.merge(yAxisG.select('.axis-label'))
    .attr('x', -innerHeight/2)
  	.text(yAxisLabel);
  
  // Here is the xAxis. 
  const xAxisGEnter = gEnter.append('g')
  	.attr('class', 'x-axis');
  const xAxisG = g.select('.x-axis');
  xAxisGEnter.merge(xAxisG)
    .call(xAxis)
  	.attr('transform', `translate(0, ${innerHeight})`)
		.select('.domain').remove();
  
  xAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 35)
      .attr('fill', 'black')
  	.merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth/2)
      .text(xAxisLabel);
  
  const lineGenerator = line()
  	.x(d => xScale(xValue(d)))
  	.y(d => yScale(yValue(d)))
  	.curve(curveBasis);
  
  const linePaths = g.merge(gEnter).selectAll('.line-path')
  	.data(nested);
  linePaths
    .enter().append('path')
      .attr('class', 'line-path')
  	.merge(linePaths)
      .attr('d', d => lineGenerator(d.values))
  		.attr('stroke', d => colorScale(d.key));
  
  const selectedYearDate = parseYear(selectedYear);
  
  gEnter
    .append('line')
      .attr('class', 'selected-year-line')
      .attr('y1', 0)
    .merge(g.select('.selected-year-line'))
      .attr('x1', xScale(selectedYearDate))
      .attr('x2', xScale(selectedYearDate))
      .attr('y2', innerHeight)
      .attr('stroke', 'black')
  
  gEnter.append('text')
      .attr('class', 'title')
      .attr('x', 260)
      .attr('y', -15)
  	.merge(g.select('.title'))
      .text(title);
  
  gEnter.append('rect')
  	.attr('class', 'mouse-interceptor')
  	.merge(g.select('.mouse-interceptor'))
  	.attr('width', innerWidth)
  	.attr('height', innerHeight)
  	.attr('fill', 'none')
  	.attr('pointer-events', 'all')
  	.on('mousemove', function() {
  		const x = mouse(this)[0];
    	const hoverDate = xScale.invert(x);
    	setSelectedYear(hoverDate.getFullYear());
  	})
}